@@ -123,6 +123,7 @@ class RobustSocketMap(ControlSocketSocketMap):
         if self.needresp:
             self.controlreceiver.send(b('i'))
             self.needresp = False
+        super(RobustSocketMap, self).close()
         
 
 
@@ -191,6 +192,9 @@ class SelectSocketMap(FragileSocketMap):
     
     def poll(self, timeout):
         """ Poll for I/O. """
+        if self.closed:
+            raise asynchia.SocketMapClosedError
+        
         interrupted = False
         
         try:
@@ -221,15 +225,18 @@ class SelectSocketMap(FragileSocketMap):
         if interrupted:
             self.do_interrupt()
     
-    def run(self):
-        """ Periodically poll for I/O. """
-        while True:
-            self.poll(None)
-    
     def close(self):
         """ See SocketMap.close """
         for handler in self.socket_list[1:]:
             self.notifier.cleanup_obj(handler)
+        
+        self.writers = []
+        self.socket_list = []
+        
+        super(SelectSocketMap, self).close()
+    
+    def is_empty(self):
+        return bool(self.socket_list)
 
 
 class PollSocketMap(RobustSocketMap):
@@ -263,8 +270,16 @@ class PollSocketMap(RobustSocketMap):
     
     def poll(self, timeout):
         """ Poll for I/O. """
+        if self.closed:
+            raise asynchia.SocketMapClosedError
+        
         interrupted = False
         
+        # Stupidest API ever. epoll accepts a float in seconds whereas
+        # poll accepts an int in millseconds.
+        if timeout is not None:
+            timeout = int(timeout * 1000)
+        
         try:
             active = self.poller.poll(timeout)
         except IOError, err:
@@ -277,7 +292,12 @@ class PollSocketMap(RobustSocketMap):
             if fileno == self.controlfd:
                 interrupted = True
                 continue
-            obj = self.socket_list[fileno]
+            try:
+                obj = self.socket_list[fileno]
+            # MacOS seems to give us invalid fds and thus we need to do
+            # this sanity check.
+            except KeyError:
+                continue
             if flags & (select.POLLIN | select.POLLPRI):
                 if obj.connected and is_closed(obj.socket):
                     self.notifier.close_obj(obj)
@@ -292,11 +312,6 @@ class PollSocketMap(RobustSocketMap):
         if interrupted:
             self.do_interrupt()
     
-    def run(self):
-        """ Periodically poll for I/O. """
-        while True:
-            self.poll(None)
-    
     def handler_changed(self, handler):
         """ Update flags for handler. """
         # self.poller.register is compatible to 2.5 whilst
@@ -325,6 +340,12 @@ class PollSocketMap(RobustSocketMap):
         RobustSocketMap.close(self)
         for handler in self.socket_list.itervalues():
             self.notifier.cleanup_obj(handler)
+        self.socket_list = {}
+        
+        super(PollSocketMap, self).close()
+    
+    def is_empty(self):
+        return bool(self.socket_list)
 
 
 class EPollSocketMap(RockSolidSocketMap):
@@ -359,6 +380,9 @@ class EPollSocketMap(RockSolidSocketMap):
     
     def poll(self, timeout):
         """ Poll for I/O. """
+        if self.closed:
+            raise asynchia.SocketMapClosedError
+        
         # While select.poll is alright with None, select.epoll expects
         # -1 for no timeout,
         if timeout is None:
@@ -393,11 +417,6 @@ class EPollSocketMap(RockSolidSocketMap):
         if interrupted:
             self.do_interrupt()
     
-    def run(self):
-        """ Periodically poll for I/O. """
-        while True:
-            self.poll(None)
-    
     def handler_changed(self, handler):
         """ Update flags for handler. """
         self.poller.modify(handler.fileno(), self.create_flags(handler))
@@ -423,6 +442,11 @@ class EPollSocketMap(RockSolidSocketMap):
         """ See SocketMap.close """
         for handler in self.socket_list.itervalues():
             self.notifier.cleanup_obj(handler)
+        self.poller.close()
+        super(EPollSocketMap, self).close()
+    
+    def is_empty(self):
+        return bool(self.socket_list)
 
 
 # It is possible to only get hangup events by applying the hack presented
@@ -510,6 +534,9 @@ class KQueueSocketMap(RockSolidSocketMap):
         )
     
     def poll(self, timeout=None):
+        if self.closed:
+            raise asynchia.SocketMapClosedError
+        
         interrupted = False
         
         try:
@@ -539,6 +566,10 @@ class KQueueSocketMap(RockSolidSocketMap):
         self.queue.close()
         for handler in self.socket_list.itervalues():
             self.notifier.cleanup_obj(handler)
+        super(KQueueSocketMap, self).close()
+    
+    def is_empty(self):
+        return bool(self.socket_list)
 
 
 DefaultSocketMap = SelectSocketMap
